#!/bin/bash

# sampl: sync_vk1_arm1
TMP=/tmp/ntpq_out$$

# sample 'ntpq -p' output:
#     remote           refid      st t when poll reach   delay   offset  jitter
#==============================================================================
#*vk1-0           LOCAL(0)         6 u   45   64  177    0.347   30.182   0.538
#+vk2-0           LOCAL(0)         6 u   44   64  177    0.337   30.240   0.518
#xvk1-1           LOCAL(0)         6 u   42   64  177    0.372   30.365   0.539
# vk2-1           LOCAL(0)         6 u   45   64  177    0.350   30.172   0.528
#
#chronyc sources
#210 Number of sources = 1
#MS Name/IP address         Stratum Poll Reach LastRx Last sample
#===============================================================================
#^* terrai.lab.sun.mcst.ru        3   6   377    50  +2988ns[+4000ns] +/-   12ms

usage()
{
	echo "
usage:
-r  Имя сервера времени, с которым проверяется синхроизация
    (колонка remote в выдае ntpq -p)
-d  Максимально допустимое расхождение времени (мс)
    текущего ВК и сервера заданного в опциии -r. По умолчаню 100 мс.
"
}
MAX_DIF=100
VK=""
OPTIONS="r:d:"
while getopts $OPTIONS option
do
	case $option in
		r) VK="$OPTARG";;
		d) MAX_DIF="$OPTARG";;
		*) usage; exit 99;;
	esac
done
for i in `seq 1 600`; do
	ntpq -p > $TMP 2>&1
    if [ $? != 0 ]; then
        chronyc sources > $TMP 2>&1
        RES_LN=`grep "^^[*+] ${VK:0:15}" $TMP | sort -r | head -n1`
    else
        RES_LN=`grep "^[*+]${VK:0:15}" $TMP | sort -r | head -n1`
    fi
	[[ ! -z "$RES_LN" ]] && break;
	sleep 1
done
ntpq -p > $TMP 2>&1
if [ $? != 0 ]; then
    chronyc sources > $TMP 2>&1
    RES_LN=`grep "^^[*+] ${VK:0:15}" $TMP | sort -r | head -n1`
    DIFF_T=`echo $RES_LN | awk 'BEGIN{FS = "[ []"}{print $7}'`
    DIFF_N=`echo $DIFF_T | awk '/ns/{print 0; exit}/us/{print 0; exit}/ms/{print $1*1; exit}/s/{print $1*1000}'`
else
    RES_LN=`grep "^[*+]${VK:0:15}" $TMP | sort -r | head -n1`
    DIFF_T="`echo $RES_LN | awk '{print $(NF-1)}'` us"
    DIFF_N=${DIFF_T%.*}
fi
if [[ -z "$RES_LN" ]]; then
	if [ -z "$VK" ]; then
		echo "Нет достоверных поставщиков времени"
	else
		echo "Сервер $VK не входит в список достоверных поставщиков времени"
	fi
	rm $TMP
	echo FAIL
	exit 0;
else
	[ -z "$VK" ] && VK=`sed -n 's/^\*\([^ \t]*\).*/\1/p' $TMP`
	echo "МАКСИМАЛЬНОЕ РАСХОЖДЕНИЕ с сервером времени $VK = $DIFF_T"
	rm $TMP
	if (( DIFF_N <= MAX_DIF && DIFF_N >= -MAX_DIF )); then
		echo PASS
		exit 0;
	else
		echo FAIL
		exit 1;
	fi
fi

