/*
 * rt_model_tc/src/getopt.c
 * Copyright (C) MCST 2015 Leonid Ananiev (leoan@mcst.ru)
 */
#include "hrt_model.h"

#ifdef USE_SCHED_DEADLINE
#define PROG_OPT	"k:NF:ER:1:b:B:t:w:l:L:D:I:m:c:d:HaYypr:hv"
#else
#define PROG_OPT	"k:NF:ER:1:b:B:t:w:l:L:D:I:m:c:d:HaYypr:hv"
#endif
char *prog_name;
int prog_argc;
char **prog_argv;

static void usage()
{
	(void)printf("Usage: %s [-%s] [args]\n", prog_name, PROG_OPT);
	(void)printf("-N prio      set Normal scheduler policy for next threads\n");
	(void)printf("-F prio      set Fifo scheduler policy and with 'prio' for next threads\n");
	(void)printf("-E prio      set dEadline scheduler policy for next threads\n");
	(void)printf("-R prio      set RR scheduler policy and with 'prio' for next threads\n");
	(void)printf("-L           cpu main work load as a percentage\n");
	(void)printf("-l           see '-L'\n");
	(void)printf("             The options -N -F -R -l -T are aplicable to the next -1 -b -B options subset.");
	(void)printf("-1 cpu       run thread waken up by interrupt on 'cpu'#\n");
	(void)printf("-b cpu       run work thread on 'cpu'# or use '-bu' for unbound. Multiple -b is permited.\n");
	(void)printf("-B cpu       as '-b' but work all previous listed requests.\n");
	(void)printf("-m rts_mask  real time mode mask\n");
	(void)printf("-v           verbose\n");
	(void)printf("-t           interrupts interval (microseconds, default 1000)\n");
        (void)printf("-I usec      max time of irq enter for test PASS\n");
        (void)printf("-D usec      max time of processes switch\n");
	(void)printf("-w           work value could be done per sec (in stable seria test run result)\n");
	(void)printf("-c comment   <comment> for current test run env\n");
	(void)printf("-k loop_num  number of loops in main wait-broadcast test\n");
	(void)printf("-d dev_path  Use 'dev_path' instead of default '/dev/mpv/mpv_0_in:0'\n");
	(void)printf("-H           Print header in res_rt_model.cvs file\n");
	(void)printf("-Y           read(mpv) without delaY (flag O_NDELAY, O_NONBLOCK).\n");
	(void)printf("-y           Wait on cpu while read(mpv)\n");
	(void)printf("-p           Use poll() for interrupt waiting\n");
	(void)printf("-r {1/0}     Set(1)/unset(0) RT_CPU mode for cpus used by rt_model (default 1).\n");
	(void)printf("args         arguments\n\n");
}

int prog_opt_atol(char *opt, char *str, long *val)
{
	int cnt;

	if (strncmp(str, "0x", 2) == 0 || strncmp(str, "-0x", 3) == 0) {
		cnt = sscanf(str, "%lx", (long *)val);
	} else {
		cnt = sscanf(str, "%lu", (long *)val);
	}

	if (cnt != 1) {
		printf("Inavlid option %s %s\n", opt, str);
		return (1);
	}

	return (0);
}

void prog_getopt(void)
{
	  char opt;

	if (prog_argc == 0) {
		printf("Invalid arguments number: exp > %d, act %d\n", 0, prog_argc);
		usage();
		exit(1);
	}
	prog_name = prog_argv[0];
	tho[0].cpu_mask = 0;
	tho[0].policy = policy;
	tho[0].policy_name = policy_name;
	tho[0].prio = prio;
	tho[0].targ_wload = targ_wload;

	while ((opt = getopt(prog_argc, prog_argv, PROG_OPT)) != EOF) {
		switch (opt) {
		case 'N':
			policy = SCHED_NORMAL;
			policy_name = "NORM";
			prio = 0;
			break;
		case 'F':
			policy = SCHED_FIFO;
			policy_name = "FIFO";
			prio = atoi(optarg);
			break;
		case 'E':
			policy = SCHED_DEADLINE;
			policy_name = "DEADLINE";
			prio = 0;
			break;
		case 'R':
			policy = SCHED_RR;
			policy_name = "RR";
			prio = atoi(optarg);
			break;
		case 'T':
			targ_omitt = atoi(optarg);
			break;
		case '1':
		case 'B':
		case 'b':
			if (optarg[0] == 'u')
				tho[thrds_num].cpu_mask = 0;
			else {
				tho[thrds_num].cpu = atoi(optarg);
				tho[thrds_num].cpu_mask = 1 << tho[thrds_num].cpu;
			}
			tho[thrds_num].policy = policy;
			tho[thrds_num].policy_name = policy_name;
			tho[thrds_num].prio = prio;
			tho[thrds_num].targ_wload = targ_wload;
			tho[thrds_num].back_grnd = (opt == 'b') || (opt == 'B');
			tho[thrds_num].forget_mis = (opt == 'b');
			thrds_num++;
			if (opt == '1')
				num_intr_thrds = thrds_num;
			break;
		case 'k':
			loop_num = atoi(optarg);
			break;
		case 'I':
			enter_pass = atoi(optarg);
			if (enter_pass < 0) {
				printf("Invalid option -I = %d\n", enter_pass);
				exit(1);
			}
			break;
		case 'D':
			max_wup_pass = atoi(optarg);
			if (max_wup_pass < 0) {
				printf("Invalid option -D = %d\n", max_wup_pass);
				exit(1);
			}
			break;
		case 'm':
			prog_opt_atol(&opt, optarg, &rts_mask);
			do_set_rts_mask = 1;
			break;
		case 'd': strcpy(devmpv_path, optarg);
			break;
		case 't':
			intrpt_intrv_usec = atoi(optarg);
			break;
		case 'l':
			targ_wload = atoi(optarg);
			break;
		case 'L':
			targ_wload = atoi(optarg);
			break;
		case 'w':
			work_by_user = atoi(optarg);
			break;
		case 'v':
			verbose = 1;
			break;
		case 'H':
			prn_header = 1;
			break;
		case 'Y':
			nodelay_read = 1;
			break;
		case 'y':
			wait_on_cpu = 1;
			break;
		case 'p':
			use_poll = 1;
			break;
		case 'c': strcpy(comment, optarg);
			break;
		case 'r': rt_cpu = !!atoi(optarg);
			break;
		case 'h':
			usage();
			exit(0);
		default:
			printf("Invalid option %c\n", opt);
			usage();
			exit(1);
		}
	}

	prog_argc -= optind;
	prog_argv += optind;
}
