#include "test.h"

#define PROG_OPT	"hP:n:m:t:bsv"
char *prog_name;
int prog_argc;
char **prog_argv;

static void usage()
{
	(void)printf("Usage: %s [-%s] [args]\n", prog_name, PROG_OPT);
	(void)printf("-P rt_policy    real time policy: SCHED_OTHER|0|SCHED_FIFO|1|SCHED_RR|2\n");
	(void)printf("-n thr_num      threads number\n");
	(void)printf("-m cpu_mask     main thread cpu mask\n");
	(void)printf("-t cpu_mask     treads cpu mask\n");
	(void)printf("-b              broadcast under mutex\n");
	(void)printf("-s              start reserv\n");
	(void)printf("-v              verbose\n");
}

static void print_prog_opt()
{
	int i;

	(void)printf("%s parameters\n", prog_name);
	(void)printf("-P rt_policy  %d, rt_policy_name %s\n", rt_thr_policy, rt_thr_policy_name);
	(void)printf("-n pthr_num   %d\n", num_threads);
	(void)printf("-m cpu_mask   %d\n", m_mask);
	(void)printf("-t cpu_mask   %d\n", t_mask);
	(void)printf("-b            %d\n", broadcast_under_mutex);
	(void)printf("-s            %d\n", start_reverse);
	(void)printf("-v            %d\n", verbose);
	for (i = 0; i < prog_argc; i++) {
		(void)printf("arg %d           %s\n", i, prog_argv[i]);
	}
}

int prog_opt_atoi(char *opt, char *str, unsigned int *val)
{
	int cnt;

	if (strncmp(str, "0x", 2) == 0 || strncmp(str, "-0x", 3) == 0) {
		cnt = sscanf(str, "%x", (unsigned *)val);
	} else {
		cnt = sscanf(str, "%u", (unsigned *)val);
	}

	if (cnt != 1) {
		printf("Inavlid option %s %s\n", opt, str);
		return (1);
	}

	return (0);
}

int prog_opt_rt_thr_policy(char *name)
{
	if (strcmp(name, "SCHED_OTHER") == 0 || strcmp(name, "0") == 0) {
		rt_thr_policy = SCHED_OTHER;
		rt_thr_policy_name = "SCHED_OTHER";
	} else if (strcmp(name, "SCHED_FIFO") == 0 || strcmp(name, "1") == 0) {
		rt_thr_policy = SCHED_FIFO;
		rt_thr_policy_name = "SCHED_FIFO";
	} else if  (strcmp(name, "SCHED_RR") == 0 || strcmp(name, "2") == 0) {
		rt_thr_policy = SCHED_RR;
		rt_thr_policy_name = "SCHED_RR";
	} else {
		printf("Invalid flag -P %s\n", name);
		return (1);
	}

	return (0);
}

void prog_getopt(void)
{
	int err = 0, opt;

	prog_name = prog_argv[0];
/*
	if (prog_argc == 0) {
		printf("Invalid arguments number: exp > %d, act %d\n", 0, prog_argc);
		usage();
		exit(1);
	}
*/
	while ((opt = getopt(prog_argc, prog_argv, PROG_OPT)) != EOF) {
		switch (opt) {
		case 'h':
			usage();
			exit(0);
		case 'P':
			if (prog_opt_rt_thr_policy(optarg)) {
				err++;
			}
			break;
		case 'n':
			if (prog_opt_atoi("-n", optarg, (unsigned *)&num_threads)) {
				err++;
			}
			break;
		case 'm':
			if (prog_opt_atoi("-m", optarg, (unsigned *)&m_mask)) {
				err++;
			}
			break;
		case 't':
			if (prog_opt_atoi("-t", optarg, (unsigned *)&t_mask)) {
				err++;
			}
			break;
		case 'b':
			broadcast_under_mutex = 1;
			break;
		case 's':
			start_reverse = 1;
			break;
		case 'v':
			verbose = 1;
			break;
		default:
			printf("Invalid option\n");
			usage();
			exit(1);
		}
	}

	prog_argc -= optind;
	prog_argv += optind;

	if (verbose) print_prog_opt();	
}

unsigned long long prog_time(void)
{
	struct timeval tv;

	if (gettimeofday(&tv, NULL)) {
		PRINT_PROG_PLACE;
		printf("gettimeofday() failed: %d (%s)",
			errno, strerror(errno));
		exit(1);
	}

	return (tv.tv_sec * 1000000ULL + tv.tv_usec);
}

static char prog_time_str[1024];

char *sprint_prog_time(unsigned long long time)
{
	unsigned int hh, mm, ss, ms, mcs;
	char str[1024];
	int all_prt_flg = 0;

	mcs = time % 1000;
	time = time / 1000;

	ms = time % 1000;
	time = time / 1000;

	ss = time % 60;
	time = time / 60;

	mm = time % 60;
	time = time / 60;

	hh = time;

	prog_time_str[0] = 0;

	if (hh) {
		sprintf(str, "%uh:", hh);
		all_prt_flg = 1;
		strcat(prog_time_str, str);
	}
	if (mm || all_prt_flg) {
		sprintf(str, "%um:", mm);
		all_prt_flg = 1;
		strcat(prog_time_str, str);
	}
	if (ss || all_prt_flg) {
		sprintf(str, "%us:", ss);
		all_prt_flg = 1;
		strcat(prog_time_str, str);
	}
	if (ms || all_prt_flg) {
		sprintf(str, "%ums:", ms);
		all_prt_flg = 1;
		strcat(prog_time_str, str);
	}
	sprintf(str, "%umcs", mcs);
	strcat(prog_time_str, str);

	return (prog_time_str);
}

void print_prog_time(unsigned long long time)
{
	printf("%s\n", sprint_prog_time(time));
}
