#include "test.h"
#ifndef _XOPEN_SOURCE
#define _XOPEN_SOURCE 500
#endif
#include <signal.h>
static long long		times[MAX_THREADS];

static  pthread_mutex_t		glob_mx;
static  pthread_cond_t		glob_cv;
static  pthread_t		all_thr[MAX_THREADS];
static  int			min_prior = 1;

int 		rt_thr_policy = SCHED_FIFO;
char		*rt_thr_policy_name = "SCHED_FIFO";
int 		num_threads = 6;
int		m_mask = 1;
int		t_mask = 1;
int 		broadcast_under_mutex = 0;
int		start_reverse =0;
int 		verbose = 0;

void set_mask(int cpu, long *mask)
{
	*mask = (0x1 << cpu);
}

void *start_func(void *arg)
{
        struct sched_param	param;
	int			i = (int)(long)arg;
	int			rval;
        struct timeval		time;
	long			mask;

	param.sched_priority = min_prior + i;
	rval = sched_setscheduler( 0, rt_thr_policy, &param);

	set_mask(t_mask, &mask);
	rval |= sched_setaffinity(0, sizeof(unsigned long), (cpu_set_t *)&mask);
	if (rval) {
		PRINT_PROG_PLACE;
		printf("Could not set priority %d policy %d or affinity 0x%lx for thead %02d",
						min_prior + i, rt_thr_policy, mask, i);
		perror("");
		printf("TEST UNRESOLVED\n");
		exit (1);
	}
	pthread_mutex_lock(&glob_mx);
	printf_v("Thread %02d started with priority %d policy %d\n",
					i + 1, min_prior + i, rt_thr_policy);
	fflush(stdout);
	rval = pthread_cond_wait(&glob_cv, &glob_mx);
	gettimeofday(&time, 0);
	times[i] = (time.tv_sec * 1000000LL + time.tv_usec);
	pthread_mutex_unlock(&glob_mx);
//	sleep(5);
	return ((void *)0);
}


int main(int argc, char **argv)
{
	long			i;
	pthread_attr_t		thr_attr0;
	int			rval;
	long long		wakeup_time, w;
	struct sched_param	param;
        struct timeval		time;
	long			mask;

	prog_argc = argc;
	prog_argv = argv;

	prog_getopt();

	if (num_threads > MAX_THREADS) {
		if (!verbose) printf("         Test uses %d threads\n", num_threads);
		printf("     Maximum possible threads = %d\n", MAX_THREADS);
		printf("TEST UNRESOLVED\n");
		return 1;
	}
        pthread_mutex_init(&glob_mx, NULL);
        pthread_cond_init(&glob_cv, NULL);
        pthread_attr_init(&thr_attr0);
        pthread_attr_setscope(&thr_attr0, PTHREAD_SCOPE_SYSTEM);

	set_mask(m_mask, &mask);
        rval = sched_setaffinity(0, sizeof(unsigned long), (cpu_set_t *)&mask);
 	if (rval) {
		PRINT_PROG_PLACE;
		printf("Could not set affinity 0x%lx for main thead 0", mask);
                perror("");
		printf("TEST UNRESOLVED\n");
                exit (1);
        }
//	(void) sigset(SIGCHLD, SIG_IGN);
	if (start_reverse) {
		for (i = num_threads - 1; i >= 0; i--) {
			rval = pthread_create(&all_thr[i], &thr_attr0,
                                start_func, (void *)i);
			if (rval) {
				PRINT_PROG_PLACE;
				perror("pthread_create error\n");
				return 1;
                	}
		}
	} else {
		for (i = 0; i < num_threads; i++) {
			rval = pthread_create(&all_thr[i], &thr_attr0,
                                start_func, (void *)i);
			if (rval) {
				PRINT_PROG_PLACE;
                        	perror("pthread_create error\n");
                        	return 1;
                	}
		}
	}
	sleep(1);
	printf_v("          ****    All threads started. Broadcast them all    ****\n");

        param.sched_priority = min_prior + num_threads;
        rval = sched_setscheduler( 0, rt_thr_policy, &param);

        if (broadcast_under_mutex) {
		pthread_mutex_lock(&glob_mx);
	}
        gettimeofday(&time, 0);
        wakeup_time = (time.tv_sec * 1000000LL + time.tv_usec);
	pthread_cond_broadcast(&glob_cv);
        gettimeofday(&time, 0);
        w = (time.tv_sec * 1000000LL + time.tv_usec);
        if (broadcast_under_mutex) {
        	pthread_mutex_unlock(&glob_mx);
	}
	sleep(2);


	rval = 0;

	printf_v("      broadcast started at        %lld\n", wakeup_time);
	printf_v("      Threads were broadcasted at %lld(%+lld)\n", w, w - wakeup_time);
	for (i = num_threads - 1; i >= 0; i--) {
		if (times[i] == 0) {
			printf("   Thread %02ld hasn't been woken up         ! ! ! ! ! \n", i + 1);
			rval = 1;
			continue;
		} else{
			printf_v("Thread %02ld is woken up at %lld(%+lld)\n", i + 1,  times[i],
				i ==  (num_threads - 1) ? times[i] - wakeup_time :  times[i] -  times[i+1]);
		}
		if (i == 0 && times[0] && wakeup_time > times[0]) {
			printf("   Thread 1 has been woken up earlier then broadcasted         ! ! ! ! ! .\n");
			rval = 1;
		}			
		if (i > 0 && times[i-1] && times[i] && times[i] > times[i-1]) {
			printf("   Thread %02ld has been woken up later then thread %ld         ! ! ! ! ! .\n",
				i + 1, i);
			rval = 1;
		}
	}

	printf("TEST %s\n", rval ? "FAILED" : "PASSED");
	sleep(3);
	return rval;
}






 
